/**
 * CMI : Cluster Method Invocation
 * Copyright (C) 2007,2008 Bull S.A.S.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Cluster.java 2438 2010-02-24 16:20:30Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.cmi.info.mapping;

import net.jcip.annotations.NotThreadSafe;

import org.ow2.cmi.info.CMIInfoExtractor;
import org.ow2.util.pool.api.IPoolConfiguration;

/**
 * Contain informations on a clustered object.
 *
 * @author The new CMI team
 * @see CMIInfoExtractor
 */
@NotThreadSafe
public final class Cluster {

    /**
     * Name of the object.
     */
    private String objectName;

    /**
     * Name of the cluster where is deployed this object.
     */
    private String clusterName = "defaultCluster";

    /**
     * True if a CMI proxy should be used to make invocations.
     */
    private boolean proxyEnabled = true;

    /**
     * Type of policy to use to access at this object.
     */
    private String policyType;

    /**
     * Type of strategy to use to access at this object. Can be null if no
     * strategy is used.
     */
    private String strategyType;

    /**
     * Properties of the policy. All the values are not casted and are instance
     * of the class String. Used only for mapping xml<->object, so not used
     * with annotation. Null if the policy doesn't support properties.
     */
    private PropertiesInfo propertiesInfo;

    /**
     * Pool's configuration.
     */
    private IPoolConfiguration poolConfiguration = null;

    /**
     * Gets the object name.
     *
     * @return the object name.
     */
    public String getObjectName() {
        return objectName;
    }

    /**
     * Sets the object name.
     *
     * @param objectName
     *            the object name to set.
     */
    public void setObjectName(final String objectName) {
        this.objectName = objectName;
    }

    /**
     * The name of the cluster.
     *
     * @return a name of the cluster where is deployed this object
     */
    public String getClusterName() {
        return clusterName;
    }

    /**
     * Sets the cluster name that contains this clustered object.
     *
     * @param clusterName
     *            the cluster name
     */
    public void setClusterName(final String clusterName) {
        this.clusterName = clusterName;
    }

    /**
     * Checks if proxy is enabled.
     *
     * @return <code>true</code> if so.
     */
    public boolean isProxyEnabled() {
        return proxyEnabled;
    }

    /**
     * Enable or disable proxy.
     *
     * @param proxyEnabled
     *            the value to set
     */
    public void setProxyEnabled(final boolean proxyEnabled) {
        this.proxyEnabled = proxyEnabled;
    }

    /**
     * @return the propertiesInfo
     */
    public PropertiesInfo getPropertiesInfo() {
        return propertiesInfo;
    }

    /**
     * @param propertiesInfo
     *            the propertiesInfo to set
     */
    public void setPropertiesInfo(final PropertiesInfo propertiesInfo) {
        this.propertiesInfo = propertiesInfo;
    }

    /**
     * @return the policyType
     */
    public String getPolicyType() {
        return policyType;
    }

    /**
     * @param policyType
     *            the policyType to set
     */
    public void setPolicyType(final String policyType) {
        this.policyType = policyType;
    }

    /**
     * @return the strategyType
     */
    public String getStrategyType() {
        return strategyType;
    }

    /**
     * @param strategyType
     *            the strategyType to set
     */
    public void setStrategyType(final String strategyType) {
        this.strategyType = strategyType;
    }

    /**
     * @return the configuration of the pool.
     */
    public IPoolConfiguration getPoolConfiguration() {
        return poolConfiguration;
    }

    /**
     * Sets the pool's configuration.
     *
     * @param poolConfiguration
     *            the given configuration
     */
    public void setPoolConfiguration(final IPoolConfiguration poolConfiguration) {
        this.poolConfiguration = poolConfiguration;
    }

    @Override
    public String toString() {
        return "ClusteredObject[" + "objectName:" + objectName
                + ",clusterName:" + clusterName + ",proxyEnabled:"
                + proxyEnabled + ",policyType:" + policyType + ",strategyType:"
                + strategyType + ",propertiesInfo:" + propertiesInfo
                + ",poolConfiguration:" + poolConfiguration + "]";
    }

    /**
     * @param indent
     * @return
     */
    public String jonasClusterToXml() {
        StringBuffer buff = new StringBuffer();
        writeValue(clusterName, buff, "<name>", "</name>");
        writeValue(policyType, buff, "<policy>", "</policy>");
        writeValue(strategyType, buff, "<strategy>", "</strategy>");
        writePoolToXml(buff, 2);
        return buff.toString();
    }

    /**
     * @param buff
     *            the String buffer to write in.
     * @param indent
     *            the ident value.
     */
    private void writePoolToXml(final StringBuffer buff, final int indent) {
        // TODO Auto-generated method stub
        buff.append(indent);
        if (poolConfiguration != null) {
            writeValue(new Integer(poolConfiguration.getMax()).toString(),
                    buff, "<max-size>", "</max-size>");
            writeValue(new Integer(poolConfiguration.getMaxWaiters())
                    .toString(), buff, "<max-waiters>", "</max-waiters>");
            writeValue(new Long(poolConfiguration.getTimeout()).toString(),
                    buff, "<timeout>", "</timeout>");
        }
    }

    /**
     * Write the xml representation of a given property.
     * @param value The value to write
     * @param buff  the Buffer
     * @param openingTag The opening tag
     * @param closingTag the closing tag
     */
    private void writeValue(final String value, final StringBuffer buff,
            final String openingTag, final String closingTag) {
        if (value != null) {
            buff.append(openingTag);
            buff.append(value);
            buff.append(closingTag);
        }
    }

    /**
     * Return indent spaces.
     *
     * @param indent
     *            number of indentation.
     * @return the indent space string.
     */
    protected String indent(final int indent) {
        String txt = "";
        for (int i = 0; i < indent; i++) {
            txt += " ";
        }
        return txt;
    }

}
