/**
 * CMI : Cluster Method Invocation
 * Copyright (C) 2007,2009 Bull S.A.S.
 * Contact: cmi@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * --------------------------------------------------------------------------
 * $Id:CMIReferenceableFactory.java 1114 2007-07-16 07:16:27Z loris $
 * --------------------------------------------------------------------------
 */

package org.ow2.cmi.pool;

import javax.naming.NamingException;

import net.jcip.annotations.ThreadSafe;

import org.ow2.cmi.controller.common.ClusterViewManager;
import org.ow2.cmi.controller.server.ProtocolNotFoundException;
import org.ow2.cmi.reference.CMIReference;
import org.ow2.cmi.reference.CMIReferenceable;
import org.ow2.cmi.reference.CMIReferenceableWrapper;
import org.ow2.cmi.reference.ObjectNotFoundException;
import org.ow2.cmi.rpc.StubOrProxyHelper;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.pool.api.PoolException;
import org.ow2.util.pool.impl.PoolEntryStatistics;
import org.ow2.util.pool.impl.PoolFactory;

/**
 * Provides methods for implementation of the pool for each lifecycle on an CMIReferencable.
 * @author The new CMI team
 */
@ThreadSafe
public class StubOrProxyFactory implements PoolFactory<CMIReferenceable<?>, CMIReference> {

    /**
     * Logger.
     */
    private static final Log LOGGER = LogFactory.getLog(StubOrProxyFactory.class);

    /**
     * Delegate to access at the cluster view.
     */
    private final ClusterViewManager clusterViewManager;

    /**
     * Expected type of the stubs/proxies.
     */
    private final Class<?> expectedType;

    /**
     * @param clusterViewManager a delegate to access at the cluster view
     */
    public StubOrProxyFactory(final ClusterViewManager clusterViewManager) {
        this.clusterViewManager = clusterViewManager;
        this.expectedType = null;
    }

    /**
     * @param clusterViewManager a delegate to access at the cluster view
     * @param expectedType expected type of the stubs/proxies
     */
    public StubOrProxyFactory(final ClusterViewManager clusterViewManager, final Class<?> expectedType) {
        this.clusterViewManager = clusterViewManager;
        this.expectedType = expectedType;
    }

    /**
     * Creates an instance of CMIReferenceable with the given hint.
     * @param cmiReference a clue given by the Pool. Could be null.
     * @throws PoolException if instance cannot be created.
     * @return the created instance.
     */
    public CMIReferenceable<?> create(final CMIReference cmiReference) throws PoolException {

        CMIReferenceable<?> stubOrProxy;
        try {
             stubOrProxy = StubOrProxyHelper.download(clusterViewManager, cmiReference);
        } catch (ProtocolNotFoundException e) {
            LOGGER.error("Cannot get a CMIReferenceable for " + cmiReference, e);
            throw new PoolException("Cannot get a CMIReferenceable for " + cmiReference, e);
        } catch (ObjectNotFoundException e) {
            LOGGER.error("Cannot get a CMIReferenceable for " + cmiReference, e);
            throw new PoolException("Cannot get a CMIReferenceable for " + cmiReference, e);
        } catch (NamingException e) {
            LOGGER.debug("No CMIReferenceable is bound for " + cmiReference, e);
            throw new NamingPoolException(
                    "No CMIReferenceable is bound for " + cmiReference, e);
        }
        if (expectedType != null && !expectedType.isAssignableFrom(stubOrProxy.getReferencedObject().getClass())) {
            LOGGER.error("Unexpected type! Required: {0} - Found: {1}",
                    expectedType.getName(), stubOrProxy.getReferencedObject().getClass().getName());
            throw new PoolException("Unexpected type! Required: " + expectedType.getName() + " - Found: "
                    + stubOrProxy.getReferencedObject().getClass().getName());
        }
        LOGGER.debug("Adding a new stub/proxy for {0}: {1}", cmiReference, stubOrProxy);
        return stubOrProxy;
    }

    /**
     * Checks if the given object with the given clue is matching.
     * @param cmiReferenceable given object against which the check should be done.
     * @param cmiReference the object used as clue to check the matching.
     * @return true if it is matching, else false.
     */
    public boolean isMatching(final CMIReferenceable<?> cmiReferenceable, final CMIReference cmiReference) {
        LOGGER.debug("{0} == {1} ?", cmiReferenceable.getReference(), cmiReference);
        return cmiReferenceable.getReference().equals(cmiReference);
    }

    /**
     * Callback called when object is gonna be removed.
     * @param cmiReferenceable that is being removed from the pool.
     */
    public void remove(final CMIReferenceable<?> cmiReferenceable) {
    }

    /**
     * Validate an instance by giving some statistics.
     * @param cmiReferenceable the instance to validate
     * @param stats some statistics to help in the validating process.
     * @return true if the element is valid, else false.
     */
    public boolean validate(final CMIReferenceable<?> cmiReferenceable, final PoolEntryStatistics stats) {
        return true;
    }

    /**
     * Sets the given clue on the given instance.
     * If no clue is used, this method shouldn't do anything
     * @param cmiReferenceable the instance to be updated with the clue
     * @param cmiRef the given clue
     * @throws PoolException if clue can't be set
     */
    public void setClue(final CMIReferenceable<?> cmiReferenceable, final CMIReference cmiRef) throws PoolException {
        if (!(cmiReferenceable instanceof CMIReferenceableWrapper)) {
            throw new PoolException("Cannot set the given clue on the instance");
        }
        // Update the reference
        ((CMIReferenceableWrapper<?>) cmiReferenceable).setReference(cmiRef);
    }

}
